#include "stdafx.h"
#include <tchar.h>
#include <strsafe.h>
#include <time.h>
#include "tray.h"
#include "onCommand.h"
#include "runUserProg.h"

//
// a sample code to unhide the target folder from the network access
// by granting the full folder access to the build-in user NETWORK
//
// Uses CACLS.exe ( that comes bundled with Windows ) to modify the folder security
// setting
//
BOOL unhideFolder( const TCHAR* folderPath )
{
	PROCESS_INFORMATION pi;
	STARTUPINFO si;
	TCHAR wzPath[MAX_PATH*2];

	StringCchPrintf( wzPath, MAX_PATH*2,
		_T("cacls \"%s\" /t /e /g network:F"),
		 folderPath );

	ZeroMemory(&si,sizeof(si));
	si.cb=sizeof(si);
	if( 0 != CreateProcess(NULL, wzPath,NULL,NULL,FALSE,NORMAL_PRIORITY_CLASS | CREATE_UNICODE_ENVIRONMENT | CREATE_NO_WINDOW,
							NULL,NULL,&si,&pi) )
	{
		CloseHandle(pi.hThread);
		WaitForSingleObject(pi.hProcess, 5000);
		CloseHandle(pi.hProcess);

		return TRUE;
	}

	return FALSE;
}

//
// a sample code to hide the target folder from the network access
// by denying the folder access to the build-in user NETWORK

// Uses CACLS.exe ( bundles with Windows ) to modify the folder security
// setting
//
BOOL hideFolder( const TCHAR* folderPath )
{
	PROCESS_INFORMATION pi;
	STARTUPINFO si;
	TCHAR wzPath[MAX_PATH*2];

	StringCchPrintf( wzPath, MAX_PATH*2,
		_T("cacls \"%s\" /t /e /d network"),
		 folderPath );

	ZeroMemory(&si,sizeof(si));
	si.cb=sizeof(si);
	if( 0 != CreateProcess(NULL, wzPath,NULL,NULL,FALSE,NORMAL_PRIORITY_CLASS | CREATE_UNICODE_ENVIRONMENT | CREATE_NO_WINDOW,
							NULL,NULL,&si,&pi) )
	{
		CloseHandle(pi.hThread);
		WaitForSingleObject(pi.hProcess, 5000);
		CloseHandle(pi.hProcess);

		return TRUE;
	}

	return FALSE;

}

void writeLog( char* msg )
{
	HANDLE fHandle = CreateFile(
	  _T("C:\\Matrix\\Util\\Log.txt"),            // t@C
	  GENERIC_WRITE,                              // ANZX[h
	  0,                                          // L[h
	  NULL,                                       // ZLeBLqq
	  OPEN_ALWAYS,                                // 쐬@
	  FILE_ATTRIBUTE_NORMAL,                      // t@C
	  NULL);									  // ev[gt@C̃nh

	if( fHandle == INVALID_HANDLE_VALUE )
		return;

	SetFilePointer( fHandle, 0, 0, FILE_END );

	DWORD written;
	WriteFile(fHandle, msg, (DWORD)strlen(msg), &written, NULL );

	CloseHandle( fHandle );
}

extern "C"
{

//
// this function is called when Matrix dongle is either plugged or unplugged
//
__declspec(dllexport) void WINAPI OnMxEvent( BOOL bConnected )
{
#ifndef NDEBUG
	writeLog("OnMxEvent\n");
#endif

	// dongle is plugged in.
	if( bConnected == TRUE )
	{
		hideFolder(_T("c:\\matrix\\util"));
	}
	// dongle is unplugged
	else if( bConnected == FALSE )
	{
		unhideFolder(_T("c:\\matrix\\util"));
	}
}

//
// this function is called when the main program starts
//
__declspec(dllexport) BOOL WINAPI OnServiceStarted(void** winMsg)
{
#ifndef NDEBUG
	writeLog("OnServiceStarted\n");
#endif

	// allocate UINT array
	*winMsg = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(UINT)*16); //new UINT[16];

	// set WinMsg routed to OnWinMsg
	((UINT*)(*winMsg))[0] = WM_CREATE;

	((UINT*)(*winMsg))[1] = MYWM_NOTIFYICON;

	((UINT*)(*winMsg))[2] = WM_COMMAND;

	// the last element must be 0
	((UINT*)(*winMsg))[3] = 0;

	// if you do not wnat any WinMsg,
	// be sure to set winMsg to NULL
//	*winMsg = NULL;

	// returning FALSE will terminate the main program
	return TRUE;
}

//
// this function is called from the main program's WM_DESTROY,
// meaning there is no need to ask for WM_DESTROY routing
//
__declspec(dllexport) void WINAPI OnServiceStopped()
{
#ifndef NDEBUG
	writeLog("OnServiceStopped\n");
#endif

}

//
// code this API just like you code WndProc
//
__declspec(dllexport) LRESULT WINAPI OnWinMsg(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifndef NDEBUG
	char buf[256];
	sprintf_s( buf, 256, "OnWinMsg %lu\n", message );	
	writeLog(buf);
#endif

	//
	// we asked for WM_CREATE, MYWM_NOTIFYICON & WM_COMMAND
	//
	switch (message)
	{
		case WM_CREATE:
			break;

		case MYWM_NOTIFYICON:
			OnNotifyIcon( hWnd, wParam, lParam );
			break;

		// all WM_COMMAND messages to the main program 
		// will come here
		case WM_COMMAND:
	    	return OnCommand( hWnd, wParam, lParam );

	}

	return 0;
}

//
// for Windows service only
// this function is called only when the main program runs as a Windows service
// When the main program runs as an application, use WM_WTSSESSION_CHANGE
//
__declspec(dllexport) void WINAPI OnSessionChange(DWORD sessionID, DWORD dwEventType)
{
#ifndef NDEBUG
	char buf[256];
	sprintf_s( buf, 256, "OnSessionChanged %lu( sessionID: %lu )\n", dwEventType, sessionID );
	writeLog(buf);
#endif

		switch(dwEventType)
		{
			// launch NotePad in the newly logged-on user session.
			// On Vista or later OS, a service is in a different
			// session from user sessions. NotePad must run in the
			// user session, not in the service session.

			case WTS_SESSION_LOGON:
				// run NotePad
				runInUserSession(_T("NotePad.exe"), sessionID );
				break;

			case WTS_SESSION_LOGOFF:
			case WTS_CONSOLE_CONNECT:
			case WTS_REMOTE_CONNECT:
				break;
			case WTS_SESSION_UNLOCK:
				showMsgInUserSession(_T("Mxpluggin"), _T("session unlocked") );
				break;

			case WTS_SESSION_LOCK:
			case WTS_CONSOLE_DISCONNECT:
			case WTS_REMOTE_DISCONNECT:
				break;
		} // switch


}

//
// for Windows service only
// this function is called only when the main program runs as a Windows service
// & when the main program is registered as a Windows service
//
__declspec(dllexport) void WINAPI OnServiceRegistered()
{
#ifndef NDEBUG
	writeLog("OnServiceRegisterd\n");
#endif

}

//
// for Windows service only
// this function is called only when the main program runs as a Windows service
// & when it is removed as a Windows service 
//
__declspec(dllexport) void WINAPI OnServiceUnregistered()
{
#ifndef NDEBUG
	writeLog("OnServiceUnregisterd\n");
#endif

}

//
// for an application only
// this function is called only when the main program runs as an application program
// & when the main program adds the task tray icon 
//
__declspec(dllexport) BOOL WINAPI OnNotifyAdd(HWND hWnd)
{
#ifndef NDEBUG
	writeLog("OnNotifyAdd\n");
#endif

	// to use a customized tray icon, trap both WM_COMMAND and MYWM_NOTIFYICON
	// to handle icon-clicked menu and dialogs

	// Adding a tray icon will start MYWM_NOTIFYICO message
	NotifyAdd(hWnd, 0);

	// must return TRUE to enable the custom tray icon
	return TRUE;

	// be sure to return FALSE to use the default task tray icon/dialog
	//return FALSE;
}

//
// for an application only
// this function is called only when the main program runs as an application program
// & when the main program deletes the tasy tray icon 
//
__declspec(dllexport) BOOL WINAPI OnNotifyDelete(HWND hWnd)
{
#ifndef NDEBUG
	writeLog("OnNotifyDelete\n");
#endif

	NotifyDelete(hWnd, 0);

	// must return TRUE to enable the custom tray icon
	return TRUE;

	// be sure to return FALSE to use the default task tray icon/dialog
//	return FALSE;
}

} // extern "C"

