﻿using System;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.IO;
using System.Text;


namespace mxplugin
{
    //
    // 1. Interface GUID must be "3A08E57A-6821-3E5E-A5B2-305616A6F218"
    // 2. Use IMxConnect interface as is
    // 3. Plugin class GUID must be "64939A9B-9995-3B36-B233-9889E5F3AC74"
    // 4. Allow Unsafe Code in Project's Build setting
    //
    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("3A08E57A-6821-3E5E-A5B2-305616A6F218")]
    public interface IMxConnect
    {
        void OnMxEvent(Boolean bConnected);
        unsafe Boolean OnServiceStarted(void** ppWinMsg);
        void OnServiceStopped();
        Int64 OnWinMsg(IntPtr hWnd, UInt32 message, UInt64 wParam, UInt64 lParam);
        void OnSessionChange(UInt32 sessionID, UInt32 eventType);
        void OnServiceRegistered();
        void OnServiceUnregistered();
        Boolean OnNotifyAdd(IntPtr hWnd);
        Boolean OnNotifyDelete(IntPtr hWnd);
    }

    [ComVisible(true)]
    [ClassInterface(ClassInterfaceType.None)]
    [Guid("64939A9B-9995-3B36-B233-9889E5F3AC74")]
    public class Plugin : IMxConnect
    {
        // Heap API flags
        const int HEAP_ZERO_MEMORY = 0x00000008;
        const UInt32 WM_CREATE = 0x0001;

        const UInt32 WTS_SESSION_LOGON = 0x5;
        const UInt32 WTS_SESSION_LOGOFF = 0x6;
        const UInt32 WTS_CONSOLE_CONNECT = 0x1;
        const UInt32 WTS_REMOTE_CONNECT = 0x3;
        const UInt32 WTS_SESSION_UNLOCK = 0x8;
        const UInt32 WTS_SESSION_LOCK = 0x7;
        const UInt32 WTS_CONSOLE_DISCONNECT = 0x2;
        const UInt32 WTS_REMOTE_DISCONNECT = 0x4;

        //for IWin32Window Interface
        // uncomment "using System.Windows.Forms"
//        NativeWindow nativeWindow = new NativeWindow();
//        Boolean WindowHandleAssigned = false;

        Boolean bServiceStopped = false;

        // Heap API functions
        [DllImport("kernel32")]
        static extern int GetProcessHeap();
        [DllImport("kernel32")]
        unsafe static extern void* HeapAlloc(int hHeap, int flags, int size);

        Boolean hideFolder(String folderPath, Boolean bHide)
        {
            String cmdArg = "";

            if (bHide == true)
                cmdArg = folderPath + " /t /e /d network";
            else
                cmdArg = folderPath + " /t /e /g network:F";

            Process p = new Process();
            p.StartInfo.FileName = "cacls.exe";
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.Arguments = cmdArg;
            p.StartInfo.RedirectStandardOutput = false;
            p.StartInfo.CreateNoWindow = true;
            p.Start();

            string output = p.StandardOutput.ReadToEnd();
            p.WaitForExit(1000);

            return true;
        }

        private void writeLog(String logText)
        {
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS"); 
            StreamWriter writer = new StreamWriter(@"C:\matrix\mxconnect.txt", true, sjisEnc);
            writer.WriteLine( logText );
            writer.Close();
        }

        public void OnMxEvent(Boolean bConnected)
        {
            writeLog("OnMxEvent\n");

            if (bServiceStopped == true)
                hideFolder("c:\\matrix\\util", false);
            else
                hideFolder("c:\\matrix\\util", bConnected);
        }

        public unsafe Boolean OnServiceStarted(void** ppWinMsg)
        {
            writeLog("OnServiceStarted\n");

            // allocate UINT array
            *ppWinMsg = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(UInt32) * 16);

            // set WinMsg routed to OnWinMsg
            ((UInt32*)*ppWinMsg)[0] = WM_CREATE;

            // the last element must be 0
            ((UInt32*)*ppWinMsg)[1] = 0;

            // if you do not want any WinMsg to be routed,
            // be sure to set winMsg to NULL
            //	*winMsg = NULL;

            // returning FALSE will terminate the main program
            return true;
        }

        public void OnServiceStopped()
        {
            writeLog("OnServiceStopped\n");

            bServiceStopped = true;
        }

        public Int64 OnWinMsg(IntPtr hWnd, UInt32 message, UInt64 wParam, UInt64 lParam)
        {
 /*
            if (WindowHandleAssigned == false)
            {
                nativeWindow.AssignHandle(hWnd);
                WindowHandleAssigned = true;
            }
*/
            //
            // we asked for WM_CREATE only in OnServiceStarted
            // and this function is called only for WM_CREATE
            // in this sample
            //
            switch (message)
            {
                case WM_CREATE:
                    writeLog("On WM_CREATE\n");
                    break;
            }

            return 0;
        }

        public void OnSessionChange(UInt32 sessionID, UInt32 dwEventType)
        {
            writeLog("OnSessionChange\n");

            switch (dwEventType)
            {
                case WTS_SESSION_LOGON:
                case WTS_SESSION_LOGOFF:
                case WTS_CONSOLE_CONNECT:
                case WTS_REMOTE_CONNECT:
                case WTS_SESSION_UNLOCK:
                case WTS_SESSION_LOCK:
                case WTS_CONSOLE_DISCONNECT:
                case WTS_REMOTE_DISCONNECT:
                    break;
            } // switch
        }

        public void OnServiceRegistered()
        {
            writeLog("OnServcieRegistered\n");
            
        }

        public void OnServiceUnregistered()
        {
            writeLog("OnServcieUnRegistered\n");
        }

        public Boolean OnNotifyAdd(IntPtr hWnd)
        {

            writeLog("OnNotifyAdd\n");
            return false;
        }

        public Boolean OnNotifyDelete(IntPtr hWnd)
        {
            writeLog("OnNotifyDelete\n");
            return false;
        }
    }
}
